@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -50,11 +50,11 @@ def _failing_bind(self, address):
     raise OSError
 
 
-async def _create_x11_process(conn, command=None,
+async def _create_x11_process(conn, command=None, x11_forwarding=True,
                               x11_display='test:0', **kwargs):
     """Create a client process with X11 forwarding enabled"""
 
-    return await conn.create_process(command, x11_forwarding=True,
+    return await conn.create_process(command, x11_forwarding=x11_forwarding,
                                      x11_display=x11_display, **kwargs)
 
 
@@ -516,6 +516,14 @@ class _TestX11(ServerTestCase):
 
             await _create_x11_process(conn, 'sleep', x11_display='test:0')
 
+    @asynctest
+    async def test_from_connect(self):
+        """Test requesting X11 forwarding on connection"""
+
+        async with self.connect(x11_forwarding=True,
+                                x11_display='text:0') as conn:
+            await conn.create_process('sleep')
+
     @asynctest
     async def test_multiple_sessions(self):
         """Test requesting X11 forwarding from two different sessions"""
@@ -595,11 +603,29 @@ class _TestX11(ServerTestCase):
         cert = ckey.generate_user_certificate(ckey, 'name', principals=['ckey'],
                                               permit_x11_forwarding=False)
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
                 await _create_x11_process(conn, 'connect l')
 
+    @asynctest
+    async def test_forwarding_ignore_failure(self):
+        """Test ignoring failure on an X11 forwarding request"""
+
+        ckey = asyncssh.read_private_key('ckey')
+        cert = ckey.generate_user_certificate(ckey, 'name', principals=['ckey'],
+                                              permit_x11_forwarding=False)
+
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
+            proc = await _create_x11_process(
+                conn, x11_forwarding='ignore_failure', x11_display='test')
+            await proc.wait()
+
+            proc = await _create_x11_process(
+                conn, x11_forwarding='ignore_failure')
+            await proc.wait()
+
     @asynctest
     async def test_invalid_x11_forwarding_request(self):
         """Test an invalid X11 forwarding request"""
